Listing 1 - Browsing More Than One Database

    #include "dbstruct.ch"
    #include "inkey.ch"

    // Key we use to move from one browse window to the other
    #define SWITCH_KEY K_F8

    // Compiler macro to convert a character string to a
    // code block while the program is running. See last month's
    // article for details

    #define COMPILE(cexpr) &("{||" + cexpr + "}")

    // Compiler macro to add all fields from the current
    // database as tbcolumns to the specified tbrowse object
    #define BROWSE_ALL(tb) ;
        aeval(dbstruct(), ;
              {|field_info| ;
               tb:addcolumn(tbcolumnnew(field_info[DBS_NAME], ;
                                        COMPILE(field_info[DBS_NAME]))) ;
              })
    
    CLEAR SCREEN

    // Open a database to browse
    USE some_dbf NEW

    // Create a tbrowse object so we can browse it
    @ 0, 0 TO 12, 79
    tb1 = tbrowsedb(1, 1, 11, 78)

    // Browse all the fields from this database
    BROWSE_ALL(tb1)

    // Open another database to browse
    USE some_other NEW
    // Create a tbrowse object so we can browse it. Make sure
    // the two tbrowse windows do not overlap
    @ 13, 0 TO 24, 79
    tb2 = tbrowsedb(14, 1, 23, 78)

    // Browse all the fields from this database
    BROWSE_ALL(tb2)

    // First display a window from this database
    DO WHILE !tb2:stabilize()
    ENDDO
    tb2:dehilite()    // Turn off hilight bar

    // Now make a "current" browse variable point to
    // the first tbrowse object
    cb = tb1
    SELECT some_dbf

    // Now enter the usual tbrowse loop, using cb as the tbrowse
    // object
    exit_requested = .F.
    DO WHILE !exit_requested
        DO WHILE nextkey() = 0 .AND. !cb:stabilize()
        ENDDO
        // We now have two tbrowse objects stable.
        // Wait for a key
        lkey = inkey(0)
        // See if this is a standard key
        IF (def_action := check_std(lkey)) != NIL
            // It is, do the usual
            eval(def_action, cb)
        ELSE
            // it is a key exception
            DO CASE
                CASE lkey = K_ESC .OR. lkey = K_ENTER
                    exit_requested = .T.

                CASE lkey = SWITCH_KEY
                    // Switch browse objects
                    cb:dehilite()  // Turn off hilight bar
                    IF cb == tb1
                        SELECT some_dbf
                        cb = tb2
                    ELSE
                        SELECT some_other
                        cb = tb1
                    ENDIF
                    cb:hilite()
            ENDCASE
        ENDIF
    ENDDO


    FUNCTION check_std(lkey)

    LOCAL rv

        DO CASE
            CASE lkey = K_DOWN;       rv = {|b| b:down()}
            CASE lkey = K_UP;         rv = {|b| b:up()}
            CASE lkey = K_PGDN;       rv = {|b| b:pagedown()}
            CASE lkey = K_PGUP;       rv = {|b| b:pageup()}
            CASE lkey = K_CTRL_PGUP;  rv = {|b| b:gotop()}
            CASE lkey = K_CTRL_PGDN;  rv = {|b| b:gobottom()}
            CASE lkey = K_RIGHT;      rv = {|b| b:right()}
            CASE lkey = K_LEFT;       rv = {|b| b:left()}
            CASE lkey = K_HOME;       rv = {|b| b:home()}
            CASE lkey = K_END;        rv = {|b| b:end()}
            CASE lkey = K_CTRL_LEFT;  rv = {|b| b:panleft()}
            CASE lkey = K_CTRL_RIGHT; rv = {|b| b:panright()}
            CASE lkey = K_CTRL_HOME;  rv = {|b| b:panhome()}
            CASE lkey = K_CTRL_END,   rv = {|b| b:panend()}
            OTHERWISE;                rv = NIL
        ENDCASE

    RETURN rv


